# -*- coding: utf-8
from django.contrib.auth import get_user_model
from django.db.models import CharField

import django_filters

from . import models


class UserFilter(django_filters.rest_framework.FilterSet):

    class Meta:
        model = get_user_model()
        fields = [
            'username',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'iexact',
                },
            },
        }


class ProfileFilter(django_filters.rest_framework.FilterSet):

    username = django_filters.CharFilter(
        name='user__username',
        lookup_expr='iexact',
    )

    class Meta:
        model = models.Profile
        fields = [
            'user',
            'user__username',
            'title',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            },
        }



# -*- coding: utf-8
from django import forms
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin as AuthUserAdmin
from django.contrib.auth.forms import UserChangeForm, UserCreationForm
from . import models


class ProfileInline(admin.StackedInline):
    model = models.Profile
    can_delete = False
    verbose_name_plural = 'Profile'
    fk_name = 'user'


class MyUserChangeForm(UserChangeForm):
    class Meta(UserChangeForm.Meta):
        model = get_user_model()


class MyUserCreationForm(UserCreationForm):

    error_message = UserCreationForm.error_messages.update({
        'duplicate_username': 'This username has already been taken.'
    })

    class Meta(UserCreationForm.Meta):
        model = get_user_model()

    def clean_username(self):
        username = self.cleaned_data["username"]
        try:
            get_user_model().objects.get(username=username)
        except get_user_model().DoesNotExist:
            return username
        raise forms.ValidationError(self.error_messages['duplicate_username'])


class MyUserAdmin(AuthUserAdmin):
    form = MyUserChangeForm
    add_form = MyUserCreationForm
    fieldsets = AuthUserAdmin.fieldsets
    list_display = ('username', 'is_superuser', 'is_active', 'last_login', 'date_joined', )
    search_fields = ['username']
    inlines = (ProfileInline, )



# -*- coding: utf-8 -*-
from django.urls import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from model_utils.models import TimeStampedModel

from .conf import settings


class Profile(TimeStampedModel):
    """User Profile"""
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        related_name="profile",
        verbose_name=_("user"),
        on_delete=models.CASCADE
    )
    title = models.CharField(max_length=255, blank=True)

    class Meta:
        verbose_name = _('Profile')
        verbose_name_plural = _('Profiles')

    def __str__(self):
        return str(self.user)

    def get_absolute_url(self):
        return reverse('users:profile-detail', kwargs={'user__username__iexact': str(self.user)})



# -*- coding: utf-8
from django.db.models.signals import post_save
from django.dispatch import receiver

from . import models
from .conf import settings


@receiver(post_save, sender=settings.AUTH_USER_MODEL)
def user_post_save(sender, **kwargs):
    """
    After User.save is called we check to see if it was a created user. If so,
    we check if the User object wants account creation. If all passes we
    create an Account object.
    We only run on user creation to avoid having to check for existence on
    each call to User.save.
    """

    # Disable post_save during manage.py loaddata
    if kwargs.get("raw", False):
        return False

    user, created = kwargs["instance"], kwargs["created"]
    disabled = getattr(user, "_disable_account_creation", not settings.CREATE_PROFILE_ON_SAVE)
    if created and not disabled:
        models.Profile.objects.create(user=user)



# -*- coding: utf-8 -*-
from rest_framework import routers

from . import viewsets


app_name = 'genomix_users'
router = routers.SimpleRouter()
router.register(r'', viewsets.UserViewSet)
router.register(r'profiles', viewsets.ProfileViewSet)

default_router = routers.DefaultRouter()
default_router.register(r'users', viewsets.UserViewSet)
default_router.register(r'user-profiles', viewsets.ProfileViewSet)

urlpatterns = default_router.urls



# -*- coding: utf-8
from django.conf import settings as default_settings
from appconf import AppConf


class GenomixUsersConf(AppConf):
    CREATE_PROFILE_ON_SAVE = True

    class Meta:
        proxy = True

    def configure_create_profile_on_save(self, value):
        return getattr(default_settings, 'CREATE_PROFILE_ON_SAVE', value)


settings = GenomixUsersConf()



# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from django.contrib.auth.validators import UnicodeUsernameValidator

from rest_framework import serializers

from . import models


class UserSerializer(serializers.ModelSerializer):
    """Serializer for Users."""

    class Meta:
        model = get_user_model()
        fields = (
            'id', 'username', 'email',
            'first_name', 'last_name',
            'is_staff', 'is_active', 'is_superuser',
            'last_login', 'date_joined',
        )
        # NOTE: This is to make it accessible to ProfileSerializer
        extra_kwargs = {
            'username': {
                'validators': [UnicodeUsernameValidator()],
            }
        }


class ProfileSerializer(serializers.ModelSerializer):
    """Serializer for User Profiles."""
    user = UserSerializer()

    class Meta:
        model = models.Profile
        fields = (
            'id', 'user', 'title',
            'created', 'modified',
        )

    def update(self, instance, validated_data):
        user_data = validated_data.pop('user')
        username = user_data.get('username')

        instance.user = get_user_model().objects.get(username=username)
        instance.title = validated_data.get('title')
        instance.save()
        return instance



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-02-09 14:49
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('title', models.CharField(blank=True, max_length=255)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='profile', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name_plural': 'Profiles',
                'verbose_name': 'Profile',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import permissions, viewsets
from rest_framework.filters import SearchFilter

from . import filters, models, serializers


class UserViewSet(viewsets.ReadOnlyModelViewSet):
    """A simple ViewSet for viewing Users."""

    queryset = get_user_model().objects.all()
    serializer_class = serializers.UserSerializer
    permission_classes = (permissions.IsAuthenticated, )
    filter_backends = (DjangoFilterBackend, SearchFilter)
    filter_class = filters.UserFilter
    lookup_field = 'username__iexact'
    search_fields = ('user__username', )


class ProfileViewSet(viewsets.ModelViewSet):
    """A simple ViewSet for viewing User Profiles."""

    queryset = models.Profile.objects.all()
    serializer_class = serializers.ProfileSerializer
    permission_classes = (permissions.IsAuthenticated, )
    filter_backends = (DjangoFilterBackend, SearchFilter)
    filter_class = filters.ProfileFilter
    lookup_field = 'user__username__iexact'
    search_fields = ('user__username', 'title')
    http_method_names = ['get', 'put', 'patch', 'options', 'head']



# -*- coding: utf-8
from django.apps import apps, AppConfig
from django.core.exceptions import ImproperlyConfigured


class GenomixUsersConfig(AppConfig):
    name = 'genomix_users'
    verbose_name = 'Users'

    requirements = [
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.staticfiles',
        'rest_framework',
        'rest_framework.authtoken',
        'rest_auth',
        'django_python3_ldap',
    ]

    def ready(self):
        from . import signals  # noqa NOTE: This is to setup signals

        for requirement in self.requirements:
            if not apps.is_installed(requirement):
                raise ImproperlyConfigured('{0} must be in INSTALLED_APPS'.format(requirement))



# -*- coding: utf-8 -*-
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import TemplateView


class AppView(LoginRequiredMixin, TemplateView):
    """Class that can be reused for projects using ReactJS front-ends

    Notes:
        - To override login URL, you can set `LOGIN_URL` in settings.py
        - To override template_name, you can use AppView.as_view(template_name='new') in urls.py
    """
    login_url = '/login/'
    redirect_field_name = 'redirect_to'
    template_name = 'index.html'



# -*- coding: utf-8
from django_python3_ldap import utils

from . import models
from .conf import settings


def genomix_search_filters(ldap_fields):
    """Filter LDAP for a given User group

    Args:
        ldap_fields (dict): Dict containing LDAP fields

    Returns:
        str: LDAP Search filter
    """
    ldap_fields["memberOf"] = settings.LDAP_AUTH_SEARCH_FILTER
    search_filters = utils.format_search_filters(ldap_fields)
    return search_filters


def sync_genomix_profile(user, ldap_attributes):
    """Sync User profile attributes for a given Users

    Args:
        user (User instance): User model
        ldap_attributes (dict): Dictionary with LDAP
    """

    if settings.CREATE_PROFILE_ON_SAVE:
        profile_fields = getattr(settings, 'LDAP_AUTH_PROFILE_FIELDS', None)

        if profile_fields:
            defaults = {}
            for key, value in profile_fields.items():
                value = ldap_attributes.get(value, [])
                if len(value) > 0:
                    defaults[key] = value[0]

            models.Profile.objects.update_or_create(
                user=user,
                defaults=defaults
            )
